/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA 
*/

/*
  Following code is based on GNOME sensors applet code hddtemp-plugin.c
  see http://sensors-applet.sourceforge.net/
*/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>

#include "psensor.h"

#define HDDTEMP_SERVER_IP_ADDRESS "127.0.0.1"
#define HDDTEMP_PORT_NUMBER 7634
#define HDDTEMP_OUTPUT_BUFFER_LENGTH 4048

struct hdd_info {
	char *name;
	int temp;
};

char *hdd_fetch()
{
	int sockfd;
	ssize_t n = 1;
	int output_length = 0;
	char *pc;

	struct sockaddr_in address;

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		fprintf(stderr, "ERROR: hdd_fetch, failed to open socket\n");
		return NULL;
	}

	address.sin_family = AF_INET;
	address.sin_addr.s_addr = inet_addr(HDDTEMP_SERVER_IP_ADDRESS);
	address.sin_port = htons(HDDTEMP_PORT_NUMBER);

	char *buffer = NULL;

	if (connect(sockfd,
		    (struct sockaddr *)&address,
		    (socklen_t) sizeof(address)) == -1) {
		fprintf(stderr,
			"ERROR: hdd_fetch, failed to open connection\n");
	} else {
		buffer = malloc(HDDTEMP_OUTPUT_BUFFER_LENGTH);

		pc = buffer;
		while ((n = read(sockfd,
				 pc,
				 HDDTEMP_OUTPUT_BUFFER_LENGTH -
				 output_length)) > 0) {

			output_length += n;
			pc = &pc[n];
		}

		buffer[output_length] = '\0';
	}

	close(sockfd);

	return buffer;
}

int str_index(char *str, char d)
{
	if (!str || *str == '\0')
		return -1;

	char *c = str;

	int i = 0;
	while (*c) {
		if (*c == d)
			return i;
		i++;
		c++;
	}

	return -1;
}

struct psensor *hdd_create_sensor(char *id, char *name)
{
	return psensor_create(id, name, SENSOR_TYPE_HDD_TEMP);
}

char *next_hdd_info(char *string, struct hdd_info *info)
{
	if (!string || strlen(string) <= 5	// at least 5 pipes
	    || string[0] != '|')
		return NULL;

	// skip first pipe
	char *c = string + 1;

	// name
	int idx_name_n = str_index(c, '|');

	if (idx_name_n == -1)
		return NULL;
	c = c + idx_name_n + 1;

	// skip label
	int i = str_index(c, '|');
	if (i == -1)
		return NULL;
	c = c + i + 1;

	// temp
	i = str_index(c, '|');
	if (i == -1)
		return NULL;
	int temp = atoi(c);
	c = c + i + 1;

	// skip unit 
	i = str_index(c, '|');
	if (i == -1)
		return NULL;
	c = c + i + 1;

	info->name = malloc(idx_name_n + 1);
	strncpy(info->name, string + 1, idx_name_n);
	info->name[idx_name_n] = '\0';

	info->temp = temp;

	return c;
}

struct psensor **hdd_psensor_list_add(struct psensor **sensors)
{
	char *hddtemp_output = hdd_fetch();

	if (!hddtemp_output)
		return sensors;

	if (hddtemp_output[0] != '|') {
		fprintf(stderr, "ERROR: wrong hdd string: %s", hddtemp_output);

		free(hddtemp_output);

		return sensors;
	}

	char *c = hddtemp_output;
	struct hdd_info info;

	struct psensor **result = sensors;

	while (c && (c = next_hdd_info(c, &info))) {
		char *id = malloc(strlen("hdd ") + strlen(info.name) + 1);
		strcpy(id, "hdd ");
		strcat(id, info.name);

		struct psensor *sensor = hdd_create_sensor(id, info.name);

		struct psensor **tmp_sensors = psensor_list_add(result, sensor);

		if (result != sensors)
			free(result);

		result = tmp_sensors;
	}

	free(hddtemp_output);

	return result;
}

void hdd_psensor_update(struct psensor **sensors, struct hdd_info *info)
{
	struct psensor **sensor_cur = sensors;

	while (*sensor_cur) {
		struct psensor *sensor = *sensor_cur;

		if (sensor->type == SENSOR_TYPE_HDD_TEMP
		    && !strcmp(sensor->name, info->name)) {

			double t = info->temp;
			int i;
			for (i = 1; i < MAX_MEASURES; i++)
				sensor->values[i - 1] = sensor->values[i];

			sensor->values[MAX_MEASURES - 1] = t;

			if (sensor->min == 0 || t < sensor->min)
				sensor->min = t;

			if (sensor->max == 0 || t > sensor->max)
				sensor->max = t;

			return;
		}

		sensor_cur++;
	}
}

void hdd_psensor_list_update(struct psensor **sensors)
{
	char *hddtemp_output = hdd_fetch();

	if (!hddtemp_output)
		return;

	if (hddtemp_output[0] == '|') {

		char *c = hddtemp_output;
		struct hdd_info info;
		info.name = NULL;
		info.temp = 0;

		while (c && (c = next_hdd_info(c, &info))) {

			hdd_psensor_update(sensors, &info);

			free(info.name);
		}
	} else {
		fprintf(stderr,
			"ERROR: wrong hdd string: %s\n", hddtemp_output);
	}

	free(hddtemp_output);
}
