/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>

#include <NVCtrl/NVCtrl.h>
#include <NVCtrl/NVCtrlLib.h>

#include <glib.h>
#include <glib/gi18n.h>

#include "psensor.h"

Display *nvidia_sensors_dpy;

int nvidia_get_sensor_temp(struct psensor *sensor)
{
	int temp;
	Bool res = XNVCTRLQueryTargetAttribute(nvidia_sensors_dpy,
					       NV_CTRL_TARGET_TYPE_GPU,
					       sensor->nvidia_id,
					       0,
					       NV_CTRL_GPU_CORE_TEMPERATURE,
					       &temp);
	if (res == True)
		return temp;
	else
		return 0;
}

struct psensor *nvidia_create_sensor(int id)
{
	char name[200];

	sprintf(name, "GPU%d", id);

	struct psensor *psensor
	    = (struct psensor *)malloc(sizeof(struct psensor));

	psensor->name = strdup(name);
	psensor->enabled = 1;
	psensor->type = SENSOR_TYPE_NVIDIA;
	psensor->nvidia_id = id;
	psensor->min = 0;
	psensor->max = 0;

	int t;
	for (t = 0; t < MAX_MEASURES; t++)
		psensor->values[t] = 0;

	psensor->id = malloc(strlen("nvidia") + 1 + strlen(name) + 1);
	sprintf(psensor->id, "nvidia %s", name);

	return psensor;
}

int nvidia_init()
{
	int event_base, error_base;
	int num_gpus;

	if (!(nvidia_sensors_dpy = XOpenDisplay(NULL))) {
		return 0;
	}

	if (XNVCTRLQueryExtension(nvidia_sensors_dpy, &event_base, &error_base)) {
		if (XNVCTRLQueryTargetCount(nvidia_sensors_dpy,
					    NV_CTRL_TARGET_TYPE_GPU,
					    &num_gpus)) {
			return num_gpus;
		}

	}

	return 0;
}

void nvidia_psensor_list_update(struct psensor **sensors)
{
	struct psensor **s_ptr = sensors;

	while (*s_ptr) {
		struct psensor *sensor = *s_ptr;

		if (sensor->type == SENSOR_TYPE_NVIDIA) {
			psensor_set_current_value
			    (sensor, nvidia_get_sensor_temp(sensor));

		}

		s_ptr++;
	}

}
