/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA 
*/

#ifndef _PSENSOR_PSENSOR_H_
#define _PSENSOR_PSENSOR_H_

#include <sensors/sensors.h>

#include "color.h"

// number of stored values
#define MAX_MEASURES 600

enum psensor_type {
	SENSOR_TYPE_TEMP = 0x01,
	SENSOR_TYPE_FAN = 0x02,
	SENSOR_TYPE_LMSENSOR_TEMP = 0x10 | SENSOR_TYPE_TEMP,
	SENSOR_TYPE_NVIDIA = 0x20 | SENSOR_TYPE_TEMP,
	SENSOR_TYPE_HDD_TEMP = 0x30 | SENSOR_TYPE_TEMP,
	SENSOR_TYPE_LMSENSOR_FAN = 0x40 | SENSOR_TYPE_FAN
};

struct psensor {
	char *name;

	char *id;

	const sensors_chip_name *iname;
	const sensors_feature *feature;

	double values[MAX_MEASURES];	// index 0 for the oldest values

	struct color *color;

	int enabled;

	int type;

	double max;
	double min;

#ifdef NVIDIA_SUPPORT
	int nvidia_id;
#endif

};

int psensor_init();
struct psensor *psensor_create(char *id, char *name, int type);
void psensor_free(struct psensor *sensor);

void psensor_list_free(struct psensor **sensors);
int psensor_list_size(struct psensor **sensors);

/*
  Return 1 if there is at least one sensor of a given type, else
  returns 0 */
int psensor_list_contains_type(struct psensor **sensors, int type);

int is_temp_type(int type);
int is_fan_type(int type);

int get_min_temp(struct psensor **sensors);
int get_max_temp(struct psensor **sensors);

int get_min_rpm(struct psensor **sensors);
int get_max_rpm(struct psensor **sensors);

void update_psensor(struct psensor *sensor);

struct psensor *create_sensor(const sensors_chip_name * chip,
			      const sensors_feature * feature);

/*
  Converts the value of a sensor to a string.

  parameter 'type' is SENSOR_TYPE_LMSENSOR_TEMP, SENSOR_TYPE_NVIDIA,
  or SENSOR_TYPE_LMSENSOR_FAN
*/
char *psensor_value_to_string(int type, double value);

struct psensor **get_all_sensors();

struct psensor **psensor_list_add(struct psensor **sensors,
				  struct psensor *sensor);

void lmsensor_psensor_list_update(struct psensor **sensors);

void psensor_set_current_value(struct psensor *sensor, double value);

#endif
