#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>

#include "psensor.h"
#include "hdd.h"
#include "nvidia.h"

#ifndef PSENSOR_VERSION
#define PSENSOR_VERSION "0.0.0"
#endif

#define PROTO_VERSION "1.0"
#define PROTO_MESSAGE_TYPE_VERSION "v"
#define PROTO_MESSAGE_TYPE_SENSOR_VALUE "u"

#define CMD_USAGE "psensor-server [port]"

void exit_on_error(char *msg)
{
	perror(msg);
	exit(EXIT_FAILURE);
}

void exit_on_msg(char *msg)
{
	fprintf(stderr, "%s\n", msg);
	exit(EXIT_FAILURE);
}

void exit_on_usage()
{
	exit_on_msg(CMD_USAGE);
}

int send_proto_version(int fd)
{
	char buffer[200];

	sprintf(buffer, "%s,%s\n", PROTO_MESSAGE_TYPE_VERSION, PROTO_VERSION);

	return send(fd, buffer, strlen(buffer), MSG_NOSIGNAL) != -1;
}

int send_sensors_info(int fd, struct psensor **sensors)
{
	char buffer[200];

#if NVIDIA_SUPPORT
	nvidia_psensor_list_update(sensors);
#endif
	lmsensor_psensor_list_update(sensors);
	hdd_psensor_list_update(sensors);

	struct psensor **sensor = sensors;

	while (*sensor) {
		sprintf(buffer,
			"%s,%.0f,%s\n",
			PROTO_MESSAGE_TYPE_SENSOR_VALUE,
			(*sensor)->values[MAX_MEASURES - 1], (*sensor)->name);

		if (send(fd, buffer, strlen(buffer), MSG_NOSIGNAL) == -1)
			return 0;

		sensor++;
	}

	return 1;
}

int main(int argc, char *argv[])
{
	struct sockaddr_in serv_addr, cli_addr;
	socklen_t sin_size = sizeof(struct sockaddr_in);

	printf
	    ("Psensor - Temperature Monitor Server - v%s by wpitchoune@gmail.com\n",
	     PSENSOR_VERSION);
	printf("Released under GPLv2 license terms, read LICENSE file\n");
	printf("http://wpitchoune.net/psensor\n");

	if (argc < 2)
		exit_on_usage();

	int port = atoi(argv[1]);

	if (!psensor_init())
		exit(EXIT_FAILURE);

	struct psensor **sensors = get_all_sensors();

	if (!sensors || !*sensors)
		exit_on_msg("no sensors detected");

	int server_fd = socket(AF_INET, SOCK_STREAM, 0);

	if (server_fd == -1)
		exit_on_error("failed to open socket");

	bzero((char *)&serv_addr, sin_size);
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = INADDR_ANY;
	serv_addr.sin_port = htons(port);

	if (bind(server_fd, (struct sockaddr *)&serv_addr, sin_size) == -1)
		exit_on_error("failed to bind socket");

	if (listen(server_fd, 5) == -1)
		exit_on_error("failed to listen socket");

	while (1) {
		int client_fd = accept(server_fd,
				       (struct sockaddr *)&cli_addr,
				       &sin_size);
		if (client_fd == -1) {
			fprintf(stderr, "failed to accept connection\n");
			continue;
		}

		if (!send_proto_version(client_fd))
			continue;

		while (send_sensors_info(client_fd, sensors))
			sleep(2);
	}

	return 0;
}
