/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA 
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <libappindicator/app-indicator.h>

#include "psensor.h"
#include "ui.h"
#include "ui_appindicator.h"

GtkWidget *ui_appindicator_get_menu(struct ui_psensor * ui)
{
	GtkWidget *menu = gtk_menu_new();

	struct psensor **s_ptr = ui->sensors;

	ui->appindicator_items
	    = malloc(psensor_list_size(ui->sensors) * sizeof(GtkWidget *));

	GtkWidget *item = gtk_menu_item_new_with_label("psensor");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);

	int i = 0;
	while (*s_ptr) {
		struct psensor *sensor = *s_ptr;

		double value = sensor->values[MAX_MEASURES - 1];

		char *strv = psensor_value_to_string(sensor->type,
						     value);
		char *label =
		    malloc(strlen(strv) + 1 + strlen(sensor->name) + 1);

		sprintf(label, "%s\t%s", strv, sensor->name);

		ui->appindicator_items[i]
		    = gtk_menu_item_new_with_label(label);

		gtk_menu_shell_append(GTK_MENU_SHELL(menu),
				      ui->appindicator_items[i]);

		free(strv);
		free(label);

		s_ptr++;
		i++;
	}

	gtk_widget_show_all(menu);

	return menu;
}

void ui_appindicator_update(struct ui_psensor *ui)
{
	int n = psensor_list_size(ui->sensors);

	int i;
	for (i = 0; i < n; i++) {
		GtkWidget *item = ui->appindicator_items[i];
		struct psensor *sensor = ui->sensors[i];

		double value = sensor->values[MAX_MEASURES - 1];

		char *strv = psensor_value_to_string(sensor->type,
						     value);

		char *label =
		    malloc(strlen(strv) + 6 + 4 + strlen(sensor->name) + 1);

		sprintf(label, "%6s     %s", strv, sensor->name);

		gtk_menu_item_set_label(GTK_MENU_ITEM(item), label);

		free(strv);
		free(label);
	}
}

void ui_appindicator_init(struct ui_psensor *ui)
{
	AppIndicator *indicator = app_indicator_new("psensor",
						    "gnome-monitor",
						    APP_INDICATOR_CATEGORY_HARDWARE);

	GtkWidget *indicatormenu = ui_appindicator_get_menu(ui);

	app_indicator_set_menu(indicator, GTK_MENU(indicatormenu));

	app_indicator_set_status(indicator, APP_INDICATOR_STATUS_ACTIVE);
}
